/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/


#include "CollectionView/collectionscene.h"

#include "CollectionModel/collectionmodel.h"
#include "CollectionModel/plsfilemodel.h"
#include "MediaItem/mediaitem.h"
#include "MediaItem/mediamimedata.h"

#include "SmartPlaylist/smartplaylist.h"

#include "Playlist/playlistmodel.h"
#include "Playlist/task_manager.h"

#include "debug.h"

#include <QGraphicsItem>

/*
********************************************************************************
*        -> queue method                                                       *
********************************************************************************
*/
void CollectionScene::enqueueTrack(bool is_new_playqueue)
{
    //Debug::debug() << " ---- enqueueTrack";
    if(!_mouseGrabbedItem) return;
    TrackGraphicItem *item = static_cast<TrackGraphicItem*>(_mouseGrabbedItem);

    if(item->media) {
        if(is_new_playqueue)
          Playqueue::instance()->clear();

        Playqueue::instance()->addMediaItem(item->media);
    }
}


void CollectionScene::enqueueAlbum(bool is_new_playqueue)
{
    //Debug::debug() << " ---- enqueueAlbum";
    if(!_mouseGrabbedItem) return;
    AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);

    QList<MEDIA::TrackPtr> list = _collectionModel->getItemChildrenTracks(item->media);

    if(!list.isEmpty()) {
        if(is_new_playqueue)
          Playqueue::instance()->clear();

        Playqueue::instance()->manager()->playlistAddMediaItems(list);
    }
}

void CollectionScene::enqueueAlbumGenre(bool is_new_playqueue)
{
    //Debug::debug() << " ---- enqueueAlbumGenre";
    if(!_mouseGrabbedItem) return;
    AlbumGenreGraphicItem *item = static_cast<AlbumGenreGraphicItem*>(_mouseGrabbedItem);

    QList<MEDIA::TrackPtr> list = _collectionModel->getAlbumChildrenTracksGenre(item->media,item->_genre);

    if(!list.isEmpty()) {
        if(is_new_playqueue)
          Playqueue::instance()->clear();

        Playqueue::instance()->manager()->playlistAddMediaItems(list);
    }
}

void CollectionScene::enqueueArtist(bool is_new_playqueue)
{
    //Debug::debug() << " ---- enqueueArtist";
    if(!_mouseGrabbedItem) return;
    ArtistGraphicItem *item = static_cast<ArtistGraphicItem*>(_mouseGrabbedItem);

    QList<MEDIA::TrackPtr> list = _collectionModel->getItemChildrenTracks(item->media);

    if(!list.isEmpty()) {
        if(is_new_playqueue)
          Playqueue::instance()->clear();

        Playqueue::instance()->manager()->playlistAddMediaItems(list);
    }
}


void CollectionScene::enqueuePlaylist(bool is_new_playqueue)
{
    if(!_mouseGrabbedItem) return;
    PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(_mouseGrabbedItem);

    QList<MEDIA::TrackPtr> list;

    // rechercher dynamique pour les smart playlists
    if(item->media->p_type == T_SMART)
    {
      list = SmartPlaylist::mediaItem( item->media->rules );
    }
    // playliste normale -> lecture dans le modele
    else {
      list = _plsFileModel->getItemChildrenTracks(item->media);
    }

    if(!list.isEmpty()) {
        if(is_new_playqueue)
          Playqueue::instance()->clear();

        Playqueue::instance()->manager()->playlistAddMediaItems(list);
    }
}

void CollectionScene::enqueueSelected(bool is_new_playqueue)
{
    //Debug::debug() << "CollectionScene::enqueueSelected";
    QList<MEDIA::TrackPtr> tracks;

    if(is_new_playqueue)
      Playqueue::instance()->clear();

    QList<QGraphicsItem*> selected_items = sortedSelectedItem();

    foreach(QGraphicsItem* gi, selected_items)
    {
      if (gi->type() == GraphicsItem::AlbumType)
      {
        AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(gi);
        tracks << _collectionModel->getItemChildrenTracks(item->media);
      }
      else if (gi->type() == GraphicsItem::AlbumGenreType)
      {
        AlbumGenreGraphicItem *item = static_cast<AlbumGenreGraphicItem*>(gi);
        tracks << _collectionModel->getAlbumChildrenTracksGenre(item->media,item->_genre);
      }
      else if (gi->type() == GraphicsItem::ArtistType)
      {
        ArtistGraphicItem *item = static_cast<ArtistGraphicItem*>(gi);
        tracks << _collectionModel->getItemChildrenTracks(item->media);
      }
      else if (gi->type() == GraphicsItem::TrackType)
      {
        TrackGraphicItem *item = static_cast<TrackGraphicItem*>(gi);
        tracks << _collectionModel->getItemChildrenTracks(item->media);
      }
      else if (gi->type() == GraphicsItem::PlaylistType)
      {
        PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(gi);

        // rechercher dynamique pour les smart playlists
        if(item->media->p_type == T_SMART)
          tracks << SmartPlaylist::mediaItem( item->media->rules );
        // playliste normale -> lecture dans le modele
        else
          tracks << _plsFileModel->getItemChildrenTracks(item->media);
      }
    } // end foreach

    if(!tracks.isEmpty()) {
        Playqueue::instance()->manager()->playlistAddMediaItems(tracks);
    }
}


/*
********************************************************************************
*        -> drag method                                                        *
********************************************************************************
*/
void CollectionScene::startAlbumsDrag(QGraphicsItem* i /* = 0 */)
{
    if(!i) // multiple albums drags
    {
      MediaMimeData* mimedata = new MediaMimeData();

      foreach(QGraphicsItem* gi, sortedSelectedItem())
      {
        AlbumGraphicItem *item = static_cast<AlbumGraphicItem *>(gi);
        mimedata->addTracks(_collectionModel->getItemChildrenTracks(item->media));
      }

      QDrag *drag = new QDrag(_parentWidget);
      drag->setMimeData(mimedata);
      drag->exec();
    }
    else // single drag
    {
        AlbumGraphicItem *item = static_cast<AlbumGraphicItem *>(i);
        item->startDrag(_parentWidget);
    }
}

void CollectionScene::startAlbumsGenreDrag(QGraphicsItem* i /* = 0 */)
{
    if(!i) // multiple albums genre drags
    {
      MediaMimeData* mimedata = new MediaMimeData();

      foreach(QGraphicsItem* gi, sortedSelectedItem())
      {
        AlbumGenreGraphicItem *item = static_cast<AlbumGenreGraphicItem *>(gi);
        mimedata->addTracks(_collectionModel->getAlbumChildrenTracksGenre(item->media,item->_genre));
      }

      QDrag *drag = new QDrag(_parentWidget);
      drag->setMimeData(mimedata);
      drag->exec();
    }
    else // single drag
    {
        AlbumGenreGraphicItem *item = static_cast<AlbumGenreGraphicItem *>(i);
        item->startDrag(_parentWidget);
    }
}

void CollectionScene::startArtistsDrag(QGraphicsItem* i /* = 0 */)
{
    if(!i) // multiple artist drags
    {
      MediaMimeData* mimedata = new MediaMimeData();

      foreach(QGraphicsItem* gi, sortedSelectedItem())
      {
        ArtistGraphicItem *item = static_cast<ArtistGraphicItem *>(gi);
        mimedata->addTracks(_collectionModel->getItemChildrenTracks(item->media));
      }

      QDrag *drag = new QDrag(_parentWidget);
      drag->setMimeData(mimedata);
      drag->exec();
    }
    else // single drag
    {
        ArtistGraphicItem *item = static_cast<ArtistGraphicItem *>(i);
        item->startDrag(_parentWidget);
    }
}


void CollectionScene::startTracksDrag(QGraphicsItem* i)
{
    if(!i) // multiple tracks drags
    {
      MediaMimeData* mimedata = new MediaMimeData();

      foreach(QGraphicsItem* gi, sortedSelectedItem())
      {
        TrackGraphicItem *item = static_cast<TrackGraphicItem *>(gi);
        mimedata->addTrack(item->media);
      }

      QDrag *drag = new QDrag(_parentWidget);
      drag->setMimeData(mimedata);
      drag->exec();
    }
    else // single drag
    {
        TrackGraphicItem *item = static_cast<TrackGraphicItem *>(i);
        item->startDrag(_parentWidget);
    }
}


void CollectionScene::startPlaylistsDrag(QGraphicsItem* i)
{
    if(!i) // multiple tracks drags
    {
      MediaMimeData* mimedata = new MediaMimeData();

      foreach(QGraphicsItem* gi, sortedSelectedItem())
      {
        PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem *>(gi);

        // rechercher dynamique pour les smart playlists
        if(item->media->p_type == T_SMART)
          mimedata->addTracks( SmartPlaylist::mediaItem( item->media->rules ) );
        // playliste normale -> lecture dans le modele
        else
          mimedata->addTracks( _plsFileModel->getItemChildrenTracks(item->media) );
      }

      QDrag *drag = new QDrag(_parentWidget);
      drag->setMimeData(mimedata);
      drag->exec();
    }
    else // single drag
    {
        PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem *>(i);
        item->startDrag(_parentWidget);
    }
}

