/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/

//! local
#include "Scrobbler/scrobblermanager.h"

#include "MediaItem/mediaitem.h"
#include "yarocksettings.h"
#include "utilities.h"
#include "global_actions.h"
#include "debug.h"


//! Qt
#include <QSettings>
#include <QString>
#include <QAction>
#include <QDateTime>


/*
********************************************************************************
*                                                                              *
*    Class ScrobblerManager                                                    *
*                                                                              *
********************************************************************************
*/
ScrobblerManager::ScrobblerManager(QObject *parent)  : QObject(parent)
{
    //Debug::debug() << "--- ScrobblerManager ->creation";
    m_player         = EnginePlayer::instance();
    m_scrobbler      = 0;
    m_engineOldState = ENGINE::STOPPED;

    settingsChanged();
}

ScrobblerManager::~ScrobblerManager()
{
    //Debug::debug() << "--- ScrobblerManager ->delete";
    if(m_scrobbler)
      delete m_scrobbler;
}



void ScrobblerManager::settingsChanged()
{
    bool enable = SETTINGS()->_useLastFmScrobbler;

    if(!enable)
    {
      disconnect(m_player, 0,this, 0);

      if(m_scrobbler) {
        delete m_scrobbler;
        m_scrobbler = 0;
      }
    }
    else if(enable)
    {
      //! start scrobbling
      Debug::debug() << "--- ScrobblerManager -> start lastfm scrobbler";
      if(m_scrobbler == 0) {
        m_scrobbler     = new LastFm::Scrobbler();
      }

      if( m_scrobbler->isAuthenticated() ) {
        connect(m_player, SIGNAL(engineStateChanged()), this, SLOT(slotStateChanged()));
        connect(m_player, SIGNAL(playingInfoChanged()), this, SLOT(trackChanged()));
      }
    }

    //! change status for send love action
    ACTIONS()->value(LASTFM_LOVE)->setEnabled(enable);
    ACTIONS()->value(LASTFM_LOVE_NOW_PLAYING)->setEnabled(enable);
}


void ScrobblerManager::trackChanged()
{
    MEDIA::TrackPtr media_track = m_player->playingTrack();

    //! Submit the old track
    if (m_scrobbler->currentTrack())
        m_scrobbler->currentTrack()->scrobble();

    if(media_track->type() != TYPE_TRACK)
      return;

    uint stamp = QDateTime::currentDateTime().toTime_t();

    LastFm::Track *track = new LastFm::Track(m_scrobbler);

    track->setArtist( media_track->artist );
    track->setTrackTitle( media_track->title );
    track->setAlbum( media_track->album );
    track->setTrackLength(media_track->duration);
    track->setGenre( media_track->genre );
    track->setTrackNumber( media_track->num );
    track->setPlaybackStart(stamp);

    //! set current track
    m_scrobbler->setCurrentTrack(track);

    //! Set "Now playing"
    track->nowPlaying();
}

void ScrobblerManager::slotStateChanged()
{
    if ( m_engineOldState == ENGINE::PAUSED && m_player->engineState() == ENGINE::PLAYING)
        if (m_scrobbler->currentTrack())
            m_scrobbler->currentTrack()->pause(false);

    if ( m_engineOldState == ENGINE::PLAYING && m_player->engineState() == ENGINE::PAUSED)
        if (m_scrobbler->currentTrack())
            m_scrobbler->currentTrack()->pause(true);

    m_engineOldState = m_player->engineState();
}



void ScrobblerManager::loveTrackRequest(QString artist, QString title)
{
    Debug::debug() << "--- ScrobblerManager :loveTrackRequest track :" << title;
    LastFm::Track *track = new LastFm::Track(m_scrobbler);
    track->setTrackTitle(title);
    track->setArtist(artist);

    track->love();
    connect(track, SIGNAL(loved()),track, SLOT(deleteLater()));
}
